setwd("/data/sPlot/users/Helge/HIDDEN")
library(tidyverse)
library(foreign)
library(gridExtra)
library(grid)
library(vegan)

source("99_HIDDEN_functions.R")

##### PART 1 ####
#### 1. traits data  #### 

traits0 <- read_delim("_data/Mesobromion/traits3_cov.txt", delim =";",  ## manually corrected vowels with umlaut
                      col_names = T, locale = locale(encoding = 'UTF-8')) %>% 
  column_to_rownames("X1") %>% 
  dplyr::select(colnames(.)[which(colSums(., na.rm=T)!=0)]) %>% 
  dplyr::select(!starts_with("BLM")) %>% 
  dplyr::select(!starts_with("ZWT")) %>% 
  dplyr::select(!starts_with("Fabaceae")) %>% 
  dplyr::select(!starts_with("BEST_")) %>% 
  dplyr::select(- any_of(c("PR_STAT_Indigen", "PR_STAT_Neophyt", "PR_STAT_Archaeophyt", "URBAN_urbanophob", "URBAN_maessig_urbanophob", 
                    "URBAN_urbanoneutral", "URBAN_maessig_urbanophil", "URBAN_urbanophil", "WUH_von", "WUH_bis", "ARL_c_I_von", "ARL_c_I_bis", 
                    "BL_ANAT_hydromorph", "WEID_V", "MAHD_V", "TRIT_V"))) %>%  #empty trait
  rownames_to_column("species") %>% 
  mutate(species0=species) %>% 
  rowwise() %>% 
  # quick and dirty clean up names
  mutate(species=gsub(pattern="_", replacement = " ", x = species)) %>% 
  mutate(species=gsub(pattern=" agg | x | spec$| agg$| s | Sec |  ", replacement=" ", x=species)) %>% 
  mutate(species=gsub(pattern=" $", replacement = "", x = species)) %>% 
  mutate(species=ifelse(is.na(word(species, 1, 2)), species, word(species, 1, 2)))
dim(traits0) #907 obs. of  75 variables:



#keep only traits with >=88 completeness
traits0 <- traits0 %>% 
  dplyr::select_if(~mean(!is.na(.)) >= 0.88) 
dim(traits0)# 902 x 67




### Transform binary traits to 0-1
traits.asym.binary <- c('LEB_F_Makrophanerophyt','LEB_F_Nanophanerophyt',
                        'LEB_F_Hemikryptophyt','LEB_F_Geophyt','LEB_F_Hemiphanerophyt','LEB_F_Therophyt',
                        'LEB_F_Hydrophyt','LEB_F_Pseudophanerophyt','LEB_F_Chamaephyt',
                        'LEB_D_plurienn_pollakanth','LBE_D_plurienn_hapaxanth','LEB_D_annuell',
                        'LEB_D_bienn','V_VER_absent','V_VER_Wurzelspross','V_VER_Ausläufer',
                        'V_VER_Rhizom','V_VER_Innovationsknopse.mit.Wurzelknolle',
                        'V_VER_Innovationsknospe.mit.Speicherwurzel','V_VER_Ausläuferknolle',
                        'V_VER_Brutsprösschen','V_VER_Fragmentation','V_VER_Turio','V_VER_Sprossknolle',
                        'V_VER_phyllogener_Spross','V_VER_Rhizompleiokorm','V_VER_Zwiebel',
                        'V_VER_Ausläuferrhizom',#'V_VER_Ausläuferzwiebel',
                        'V_VER_Bulbille',
                        'ROS_T_rosettenlose.Pflanzen','ROS_T_Halbrosettenpflanze','ROS_T_Ganzrosettenpflanzen',
                        'BL_AUSD_immergrün','BL_AUSD_sommergrün','BL_AUSD_vorsommergrün',
                        'BL_AUSD_überwinternd_grün','BL_ANAT_skleromorph','BL_ANAT_mesomorph',
                        'BL_ANAT_hygromorph',#'BL_ANAT_hydromorph',
                        'BL_ANAT_blattsukkulent','BL_ANAT_helomorph',
                        'BL_FORM_gelappt_gefiedert_gefingert_mehrfach','BL_FORM_Vollblatt_Normalblatt',
                        'BL_FORM_Grasblatt_Langblatt','BL_FORM_nadelförmig','BL_FORM_röhrig',
                        'BL_FORM_schuppenförmig','BL_FORM_schwertförmig','REPR_T_Samen_Sporen',
                        'REPR_T_vegetativ','BLU_KL_WIND','BLU_KL_POLLEN',
                        'BLU_KL_NEKTAR_HONIG_INSEKTEN','STRAT_T_C','STRAT_T_CR','STRAT_T_CS',
                        'STRAT_T_CSR','STRAT_T_R','STRAT_T_S','STRAT_T_SR')

traits0 <- traits0 %>% 
  mutate_at(.vars=vars(all_of(traits.asym.binary)), 
            .funs=~(.>0)*1)


## Import traits from TRY and match to species
load("/data/sPlot2.0/TRY.all.mean.sd.3.by.genus.species.tree.Rdata")
alltry <- TRY.all.mean.sd.3.by.genus.species.tree %>% 
  dplyr::select(!ends_with(".sd")) %>% 
  dplyr::select(StandSpeciesName, LeafArea.mean:Wood.vessel.length.mean) %>% 
  dplyr::select(-Wood.vessel.length.mean, -StemDens.mean, -Stem.cond.dens.mean) %>% 
  rename_all(.funs=~gsub(pattern=".mean$", replacement="", x=.))

all.traits <- traits0 %>% 
  ungroup() %>% 
  #dplyr::select(species, species0) %>% 
  left_join(alltry %>% 
              rename(species=StandSpeciesName), 
            by="species") 
traits <- all.traits %>% 
  filter(!is.na(LeafArea))
dim(all.traits) #[1] 902  82
dim(traits) #[1] 801  82



##### 2. Header Data #### 
env0 <- read_delim("_data/Mesobromion/GVRD_MES2_site.csv", delim = ",")
str(env0) #6868 obs. of  6 variables:

set.seed(1984)
header   <- "/data/sPlot/users/Francesco/Project_11/Germany/_data/tvhabita.dbf"
env  <- env0 %>% 
  left_join(foreign::read.dbf(header) %>% 
              as.data.frame() %>% 
              dplyr::select(RELEVE_NR, LAT, LON),
            by="RELEVE_NR") %>% 
  filter(!is.na(LAT)) %>%
  filter(!(LAT==0 | LON==0)) 

env.all <- env



### 3. Import species data #### 
# columns in species correspond to those in env
# there is no PlotObservationID (yet)
species0 <- read_csv("_data/Mesobromion/GVRD_Mes2_proz2.csv", locale = locale(encoding = 'latin1'))
dim(species0) #6868 obs. of  903 variables:
rownames(species0) <- env0$RELEVE_NR

## select only plots already selected in env
species <- env %>% 
  dplyr::select(RELEVE_NR) %>% 
  left_join(species0 %>%
              mutate(RELEVE_NR=env0$RELEVE_NR), 
            by="RELEVE_NR") %>% 
  column_to_rownames("RELEVE_NR") %>% 
  ## delete species not appearing in any plot
  dplyr::select(colnames(.)[which(colSums(.)!=0)])
#dplyr::select(traits$species0)

dim(species) # [1] 5810 877

releve08trait <- species %>% 
  rownames_to_column("RELEVE_NR") %>% 
  reshape2::melt(.id="RELEVE_NR") %>% 
  rename(species0=variable, pres=value) %>% 
  as.tbl() %>% 
  filter(pres>0) %>% 
  arrange(RELEVE_NR)  %>% 
  ## attach traits 
  left_join(traits %>% 
              dplyr::select(-species), by="species0") %>% 
  mutate_at(.vars = vars(LEB_F_Makrophanerophyt:Disp.unit.leng), 
            .funs = list(~if_else(is.na(.),0,1) * pres)) %>%
  group_by(RELEVE_NR) %>%
  summarize_at(.vars= vars(LEB_F_Makrophanerophyt:Disp.unit.leng),
               .funs = list(~mean(.))) %>%
  dplyr::select(RELEVE_NR, order(colnames(.))) %>%
  reshape2::melt(id.vars="RELEVE_NR", value.name="trait.coverage") %>% 
  group_by(RELEVE_NR) %>% 
  summarize(ntraits08=mean(trait.coverage>=0.8)) %>% 
  #select only those releves where we have a coverage of >0.8 for all traits
  filter(ntraits08==1) %>% 
  pull(RELEVE_NR)

set.seed(1984)
releve08trait.samp <- sample(releve08trait, round(length(releve08trait)/10), replace=F)
species <- species %>% 
  rownames_to_column("RELEVE_NR") %>% 
  filter(RELEVE_NR %in% releve08trait.samp) %>% 
  #as.tbl() %>% 
  dplyr::select(RELEVE_NR, one_of(traits$species0)) %>% 
  column_to_rownames("RELEVE_NR") %>% 
  dplyr::select(colnames(.)[which(colSums(.)!=0)])

env <- env %>% 
  filter(RELEVE_NR %in% releve08trait.samp)

traits <- traits %>% 
  dplyr::select(-species) %>% 
  dplyr::select(species0, everything()) %>% 
  filter(species0 %in% colnames(species))


recode.traits <- function(x){
  ## recode traits to numeric
  #recode binary traits to nominal
  
  robust.mean <- function(x1,x2=NA,x3=NA,x4=NA){
    x <- c(x1,x2,x3,x4)
    if(any(!is.na(x))){mean(x, na.rm=T)} else {NA}
  }
  colnames(x)[which(colnames(x)=="LBE_D_plurienn_hapaxanth")] <- "LEB_D_plurienn_hapaxanth"
  x <- x %>% 
    mutate(BLU_KL_NEKTAR_HONIG_INSEKTEN=replace(BLU_KL_NEKTAR_HONIG_INSEKTEN, 
                                                list=species0 %in% c("Convallaria_majalis", "Maianthemum_bifolium"),
                                                values=0))
  
  x <- x %>% 
    as.tbl() %>% 
    dplyr::select(-starts_with("BL_FORM"), -starts_with("REPR_T"), -starts_with("BLU_KL"), -starts_with("STRAT_T"), -starts_with("BL_AUSD")) %>% 
    left_join(x %>% 
                dplyr::select(species0, `BL_AUSD_immergrün`:`BL_AUSD_überwinternd_grün`, REPR_T_Samen_Sporen:STRAT_T_SR) %>% 
                gather(key=Trait, value="value", -species0) %>% 
                separate(Trait, into = c("Trait", "Organ", "Level"), sep = "_", extra = "merge") %>% 
                unite(Trait, Trait, Organ) %>% 
                filter(value==1) %>% 
                dplyr::select(-value) %>% 
                spread(Trait, Level) %>% 
                mutate_at(.vars=vars(BL_AUSD:STRAT_T), 
                          .funs=~as.factor(.)), 
              by="species0")
  
  out <- x %>% 
    dplyr::select(-starts_with("BL_ANAT"), -starts_with("LEB_D"), -starts_with("ROS_T")) %>% 
    left_join(x %>% 
                dplyr::select(species0, starts_with("BL_ANAT")) %>% 
                mutate(BL_ANAT_helomorph=ifelse(BL_ANAT_helomorph==1, 1, NA)) %>% 
                mutate(BL_ANAT_hygromorph=ifelse(BL_ANAT_hygromorph==1, 2, NA)) %>% 
                mutate(BL_ANAT_mesomorph=ifelse(BL_ANAT_mesomorph==1, 3, NA)) %>% 
                mutate(BL_ANAT_skleromorph=ifelse(BL_ANAT_skleromorph==1, 4, NA)) %>% 
                rowwise() %>% 
                mutate(BL_ANAT=robust.mean(BL_ANAT_helomorph, BL_ANAT_hygromorph, BL_ANAT_mesomorph, BL_ANAT_skleromorph)) %>% 
                ungroup() %>% 
                dplyr::select(species0, BL_ANAT, BL_ANAT_blattsukkulent), 
              by="species0") %>% 
    left_join(x %>% 
                dplyr::select(species0, starts_with("LEB_D"))  %>%
                rowwise() %>% 
                mutate(LEB_D_plurienn=max(LEB_D_plurienn_pollakanth + LEB_D_plurienn_hapaxanth, na.rm=T)) %>% 
                ungroup() %>% 
                mutate(LEB_D_plurienn=ifelse(LEB_D_plurienn==1, 3, NA)) %>% 
                mutate(LEB_D_annuell=ifelse(LEB_D_annuell==1, 1, NA)) %>% 
                mutate(LEB_D_bienn =ifelse(LEB_D_bienn==1, 2, NA)) %>% 
                rowwise() %>% 
                mutate(LEB_D=robust.mean(LEB_D_annuell, LEB_D_bienn, LEB_D_plurienn)) %>% 
                ungroup() %>% 
                dplyr::select(species0, LEB_D), 
              by="species0") %>% 
    left_join(x %>% 
                dplyr::select(species0, starts_with("ROS_T")) %>% 
                mutate(ROS_T=ROS_T_Ganzrosettenpflanzen) %>% 
                mutate(ROS_T=replace(ROS_T, 
                                     list=ROS_T_Halbrosettenpflanze==1, 
                                     values=0.5)) %>% 
                mutate(ROS_T=replace(ROS_T, 
                                     list=ROS_T_rosettenlose.Pflanzen==1, 
                                     values=0)) %>% 
                dplyr::select(species0, ROS_T), 
              by="species0") 
  return(out)
}
traits <- recode.traits(traits)
 

### ordered factors


dim(species) #581 509
dim(traits) #509 54
dim(env) #581 8




######4.  Extract Environmental Factors ######
### CHELSA
library(raster)
library(sp)
Temp <- raster("../../Francesco/Ancillary_Data/CHELSA/CHELSA_bio10_01.tif")
Prec <- raster("../../Francesco/Ancillary_Data/CHELSA/CHELSA_bio10_12.tif")
PHIPHOX <- raster("../../Francesco/Ancillary_Data/ISRIC/PHIHOX_M_sl2_250m_ll.tif")
ORCDRC <- raster("../../Francesco/Ancillary_Data/ISRIC/ORCDRC_M_sl2_250m_ll.tif")


env.sp <- SpatialPointsDataFrame(coords=env %>% dplyr::select(LON, LAT), 
                                 data=env %>% dplyr::select(-LON, -LAT), 
                                 proj4string = raster::crs("+proj=longlat +datum=WGS84 +no_defs")) 
env <- env %>% 
  mutate(Temp=raster::extract(Temp, env.sp)/10) %>% 
  mutate(Prec=raster::extract(Prec, env.sp)) %>% 
  mutate(PHIPHOX=raster::extract(PHIPHOX, env.sp)/10) %>% 
  mutate(ORCDRC=raster::extract(ORCDRC, env.sp)) 



## Select only plots where >90% of species have trait info [TRY]
# releve08trait <- species %>% 
#   rownames_to_column("RELEVE_NR") %>% 
#   reshape2::melt(.id="RELEVE_NR") %>% 
#   rename(species0=variable, pres=value) %>% 
#   filter(pres>0) %>% 
#   arrange(RELEVE_NR)  %>% 
#   ## attach traits 
#   left_join(traits %>% 
#               dplyr::select(-species, LeafArea.mean), by="species0") %>% 
#   group_by(RELEVE_NR) %>% 
#   summarize(trait.coverage=mean(!is.na(LeafArea.mean))) %>% 
#   filter(trait.coverage<0.8) %>% 
#   pull(RELEVE_NR)
# 

species <- species %>% 
  rownames_to_column("RELEVE_NR")

### 5. Transform and export data 

## presence absence
species.pa <- species
species.pa[,-1] <- (species[,-1]>0)*1
## sqrt transform cov values
species.cov <- species
species.cov[,-1] <- sqrt(species.cov[,-1])
#transform percentage cover to relative.cover
species.cov <- species.cov %>% 
  mutate(sumVar = rowSums(.[-1])) %>% 
  mutate_at(.vars=vars(-RELEVE_NR), 
            .funs=~./sumVar) %>% 
  dplyr::select(-sumVar) 
dim(species.cov) #[1] 581 510


## export
write_delim(species.pa, path="_data/Mesobromion/species.v2.10perc.pa.txt", delim="\t")
write_delim(species.cov, path="_data/Mesobromion/species.v2.10perc.cov.txt", delim="\t")
write_delim(traits, path="_data/Mesobromion/traits.v2.10perc.txt", delim="\t")
write_delim(env, path="_data/Mesobromion/env.v2.10perc.cov.txt", delim="\t")

### create list of relevées
write_delim(species %>% 
              dplyr::select(RELEVE_NR), 
            path="_data/Mesobromion/ReleveList.txt", delim="\t")



##check for species without trait info
traits %>% 
  filter_at(.vars=vars(-"species0"), 
            all_vars(is.na(.))) %>% 
  nrow() ## [1] no species with no trait info ##
## simply because those 7 species without TRY data were excluded already

traits %>% 
  filter_at(.vars=vars(-"species0"), 
            any_vars(is.na(.))) %>% 
  nrow() ## [1] 109 # species with no at least 1 NA in traits


#### CORRELATION BETWEEN FUZZY WEIGHTED AND BEALS MATRICES
#### WAS RUN IN THE CLUSTER WITH THE SCRIPT 01b_MesobromionCluster.R







### PART 2 ####
source("01b_MesobromionCluster.R")
#### 1. Traits individually significant for COVER data####
traits <- read_delim("_data/Mesobromion/traits.out.10perc.cov.txt", delim="\t")
myfilelist <- list.files(path="_derived/Mesobromion/Cover", pattern="HIDDENproz_[0-9]+_.RData", full.names = T)
dataFiles = purrr::map(myfilelist, function(x){get(load(x))})
corXY = bind_rows(dataFiles) %>% 
  as_tibble()
rm( dataFiles)

trait.labs <- data.frame(Trait.comb=as.character(1:(ncol(traits)-1)), 
                         trait.name=colnames(traits)[-1]) 
                
corXY.ci <- get.ci(corXY)
corXY.ci <- corXY.ci %>% 
  arrange(desc(sign_plus), desc(Coef.obs)) %>% 
  left_join(trait.labs, by="Trait.comb") %>% 
  dplyr::select(-Test)

## NO significant TRAITS when using Cover values
# traits.sign.alone <- corXY.ci %>% 
#   filter(sign_plus) %>% 
#   pull(trait.name)
# 
# traits.sign <- traits %>% 
#   dplyr::select(species0, any_of(traits.sign.alone))
# write_delim(traits.sign, path="_data/Mesobromion/traits.out.10perc.cov.sign.txt", delim="\t")

"# A tibble: 50 x 11
   Trait.comb Coef.obs Coef.perm  q025  q975 greater.than.perm     n sign_plus sign_minus ntraits trait.name           
   <chr>         <dbl>     <dbl> <dbl> <dbl>             <dbl> <int> <lgl>     <lgl>        <int> <fct>                
 1 17            0.335     0.205 0.275 0.436             0.903   999 FALSE     FALSE            1 V_VER_Fragmentation  
 2 12            0.255     0.139 0.235 0.315             0.690   999 FALSE     FALSE            1 V_VER_Rhizom         
 3 11            0.226     0.170 0.218 0.279             0.533   999 FALSE     FALSE            1 V_VER_Ausläufer      
 4 5             0.199     0.340 0.194 0.247             0.502   999 FALSE     FALSE            1 LEB_F_Hemiphanerophyt
 5 2             0.198     0.232 0.191 0.247             0.447   999 FALSE     FALSE            1 LEB_F_Nanophanerophyt
 6 19            0.198     0.217 0.193 0.245             0.539   999 FALSE     FALSE            1 V_VER_Sprossknolle   
 7 44            0.194     0.171 0.199 0.242             0.299   999 FALSE     FALSE            1 BLU_KL               
 8 45            0.192     0.188 0.187 0.238             0.526   999 FALSE     FALSE            1 REPR_T               
 9 10            0.191     0.187 0.187 0.240             0.366   999 FALSE     FALSE            1 V_VER_Wurzelspross   
10 42            0.190     0.235 0.192 0.236             0.261   999 FALSE     FALSE            1 Disp.unit.leng       
# … with 40 more rows"

#### 2. Traits individually significant for Presence|absence data####
traits <- read_delim("_data/Mesobromion/traits.out.10perc.txt", delim="\t")
myfilelist <- list.files(path="_derived/Mesobromion/PresAbs", pattern="HIDDENpa_[0-9]+_.RData", full.names = T)
dataFiles = purrr::map(myfilelist, function(x){get(load(x))})
corXY = bind_rows(dataFiles) %>% 
  as_tibble()
rm( dataFiles)

trait.labs <- data.frame(Trait.comb=as.character(1:(ncol(traits)-1)), 
                         trait.name=colnames(traits)[-1]) 

corXY.ci <- get.ci(corXY)
corXY.ci <- corXY.ci %>% 
  arrange(desc(sign_plus), desc(Coef.obs)) %>% 
  left_join(trait.labs, by="Trait.comb") %>% 
  dplyr::select(-Test)

"# A tibble: 52 x 11
   Trait.comb Coef.obs Coef.perm  q025  q975 greater.than.perm     n sign_plus sign_minus ntraits trait.name           
   <chr>         <dbl>     <dbl> <dbl> <dbl>             <dbl> <int> <lgl>     <lgl>        <int> <fct>                
 1 35            0.300    0.0714 0.270 0.350             0.993   999 TRUE      FALSE            1 PlantHeight          
 2 34            0.279    0.201  0.250 0.326             0.993   999 TRUE      FALSE            1 LeafP                
 3 31            0.253    0.165  0.231 0.303             0.987   999 TRUE      FALSE            1 SLA                  
 4 37            0.244    0.0847 0.218 0.293             0.986   999 TRUE      FALSE            1 Seed.length          
 5 33            0.243    0.0984 0.218 0.290             0.994   999 TRUE      FALSE            1 LeafN                
 6 32            0.240    0.0846 0.218 0.290             0.986   999 TRUE      FALSE            1 LeafC.perdrymass     
 7 44            0.234    0.145  0.208 0.288             0.980   999 TRUE      FALSE            1 Disp.unit.leng       
 8 2             0.234    0.263  0.216 0.283             0.593  1998 FALSE     FALSE            1 LEB_F_Nanophanerophyt
 9 5             0.233    0.267  0.214 0.287             0.571   999 FALSE     FALSE            1 LEB_F_Hemiphanerophyt
10 30            0.227    0.177  0.203 0.281             0.970   999 FALSE     FALSE            1 LeafArea             
# … with 42 more rows"


traits.sign.alone <- corXY.ci %>% 
  filter(sign_plus) %>% 
  pull(trait.name)

traits.sign <- traits %>% 
  dplyr::select(species0, any_of(traits.sign.alone))
write_delim(traits.sign, path="_data/Mesobromion/traits.out.10perc.sign.txt", delim="\t")

